/********************************************************************
 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the "Company") for its PIC(R) Microcontroller is intended and
 supplied to you, the Company's customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *******************************************************************/

/** INCLUDES *******************************************************/
#include <system.h>

#include <stdint.h>
#include <string.h>
#include <stddef.h>

#include <usb/usb.h>
#include <app_device_cdc_basic.h>
#include <usb_config.h>

void InitializeADC();
void StartADC();

static uint8_t readBuffer[CDC_DATA_OUT_EP_SIZE];
static uint8_t writeBuffer[CDC_DATA_IN_EP_SIZE];

 BYTE adcBuffer[adc_buffer_size];
 BYTE adcBuffer2[adc_buffer_size];
 bool isLowBufferWrite = true;
 bool isLowBufferRead = true;
 bool isRunning = false;

int writeIndex = 0;
int readIndex =0;
int adc_counter =0;

/*********************************************************************
* Function: void APP_DeviceCDCBasicDemoInitialize(void);
*
* Overview: Initializes the demo code
*
* PreCondition: None
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCDCBasicDemoInitialize()
{
    CDCInitEP();

    
    line_coding.bCharFormat = 0;
    line_coding.bDataBits = 8;
    line_coding.bParityType = 0;
    //line_coding.dwDTERate = 19200;
    line_coding.dwDTERate = 115200;
}

/*********************************************************************
* Function: void APP_DeviceCDCBasicDemoTasks(void);
*
* Overview: Keeps the demo running.
*
* PreCondition: The demo should have been initialized and started via
*   the APP_DeviceCDCBasicDemoInitialize() and APP_DeviceCDCBasicDemoStart() demos
*   respectively.
*
* Input: None
*
* Output: None
*
********************************************************************/
void APP_DeviceCDCBasicDemoTasks()
{
    /* Make sure that the CDC driver is ready for a transmission. */
    if(mUSBUSARTIsTxTrfReady() == true)
    {
        int bytesToRead = adc_counter;
        uint_fast16_t i=0;
        if(bytesToRead > adc_buffer_size+adc_buffer_size)
        {
//            for(i=0;i<640;i+=2)
//            {
//                adcBuffer[i] = 2;
//                adcBuffer[i+1] = 0;
//
//                adcBuffer2[i] = 2;
//                adcBuffer2[i+1] = 0;
//
//            }
//
//            putUSBUSART(adcBuffer, 256);
            readIndex=0;
            writeIndex =0;
            adc_counter=0;
        }
        else if(bytesToRead>0 && isRunning)
        {
            if(readIndex == adc_buffer_size)
            {
                readIndex =0;
                isLowBufferRead = !isLowBufferRead;
            }
            uint8_t* pBuffer;
            if(isLowBufferRead)
            {
                pBuffer= &adcBuffer[readIndex];
            }
            else
            {
                pBuffer= &adcBuffer2[readIndex];
            }
            
            if(readIndex + bytesToRead < adc_buffer_size)
            {
                putUSBUSART(pBuffer, bytesToRead);
                readIndex+= bytesToRead;
            }
            else
            {
                bytesToRead = adc_buffer_size - readIndex;
                if(bytesToRead >0)
                {
                    putUSBUSART(pBuffer, bytesToRead);
                }
                readIndex=0;
            }
            adc_counter -= bytesToRead;
        }
    }

    /* Check to see if there is a transmission in progress, if there isn't, then
     * we can see about performing an echo response to data received.
     */
    if( USBUSARTIsTxTrfReady() == true)
    {
        uint8_t i;
        uint8_t numBytesRead;

        numBytesRead = getsUSBUSART(readBuffer, sizeof(readBuffer));

        /* For every byte that was read... */
        for(i=0; i<numBytesRead; i++)
        {
            switch(readBuffer[i])
            {
                /* '0' = stop */
                case 0x30:
                    isRunning = false;
                    break;
                /* '1' = start */
                case 0x31:
                    isRunning = true;
                    InitializeADC();
                    StartADC();
                    break;
                default:
                    break;
            }
        }

        if(numBytesRead > 0)
        {
            /* After processing all of the received data, we need to send out
             * the "echo" data now.
             */
            putUSBUSART(writeBuffer,numBytesRead);
        }
    }

    CDCTxService();
}


void APP_DeviceReadPot()
{
    if(isRunning)
    {
        if(writeIndex > adc_buffer_size - 2 || writeIndex < 0)
        {
            writeIndex = 0;
            isLowBufferWrite = !isLowBufferWrite;
        }
        if(isLowBufferWrite)
        {
            adcBuffer[writeIndex++] = ADRESH; //Measured analog voltage MSB
            adcBuffer[writeIndex++] = ADRESL; //Measured analog voltage LSB
        }
        else
        {
            adcBuffer2[writeIndex++] = ADRESH; //Measured analog voltage MSB
            adcBuffer2[writeIndex++] = ADRESL; //Measured analog voltage LSB
        }
        adc_counter += 2;
    }
}

void InitializeADC()
{
    adc_counter = 0;
    writeIndex =0;
    readIndex =0;
    isLowBufferRead = true;
    isLowBufferWrite = true;

    /*
     1. Configure the A/D module:
        • Configure analog pins, voltage reference and digital I/O (ADCON1)
        • Select A/D input channel (ADCON0)
        • Select A/D acquisition time (ADCON2)
        • Select A/D conversion clock (ADCON2)
        • Turn on A/D module (ADCON0)
    2. Configure A/D interrupt (if desired):
        • Clear ADIF bit
        • Set ADIE bit
        • Set GIE bit
    3. Wait the required acquisition time (if required).
    4. Start conversion:
        • Set GO/DONE bit (ADCON0 register)
    5. Wait for A/D conversion to complete, by either:
        • Polling for the GO/DONE bit to be cleared
        OR
        • Waiting for the A/D interrupt
    6. Read A/D Result registers (ADRESH:ADRESL);
        • clear bit ADIF, if required.
    7. For next conversion, go to step 1 or step 2, as required.
    The A/D conversion time per bit is defined as T AD.
        • A minimum wait of 3 T AD is required before the next acquisition starts.
     */

    PIR1bits.ADIF = 0x00;
    PIE1bits.ADIE = 0x00;
    IPR1bits.ADIP = 0x00;
    TRISAbits.TRISA0 = 1;
    ADCON0 = 1;
    ADCON1 = 0x0E;              // AN0 is analog input
    ADCON2bits.ADFM = 1;        // ADFM : right justified

//48 Mhz
    ADCON2bits.ADCS2 =1; // 0b110;    // 48Mhz => Tad = 64Tosc => ADCS2...0 = 110b;
    ADCON2bits.ADCS1 =1; // 0b110;    // 48Mhz => Tad = 64Tosc => ADCS2...0 = 110b;
    ADCON2bits.ADCS0 =0; // 0b110;    // 48Mhz => Tad = 64Tosc => ADCS2...0 = 110b;
    ADCON2bits.ACQT2 =1; // 0b101;    8 T AD
    ADCON2bits.ACQT1 =0; // 0b101;    8 T AD
    ADCON2bits.ACQT0 =0; // 0b101;    8 T AD

//20 Mhz
//    ADCON2bits.ADCS2 =1; // 0b110;    // 20Mhz => Tad = 16Tosc => ADCS2...0 = 101b;
//    ADCON2bits.ADCS1 =0; // 0b110;    // 20Mhz => Tad = 16Tosc => ADCS2...0 = 101b;
//    ADCON2bits.ADCS0 =1; // 0b110;    // 20Mhz => Tad = 16Tosc => ADCS2...0 = 101b;
//    ADCON2bits.ACQT2 =1; // 0b101;    12 T AD
//    ADCON2bits.ACQT1 =0; // 0b101;    12 T AD
//    ADCON2bits.ACQT0 =1; // 0b101;    12 T AD

    ADCON0bits.ADON = 1; // Turn on A/D module (ADCON0)

    PIR1bits.ADIF = 0;   //Clear ADIF bit
    PIE1bits.ADIE = 1;   //Set ADIE bit
    INTCONbits.GIEH = 1; // Set GIE bit
    ADCON0bits.GO = 1;
}

#define _XTAL_FREQ 20000000
void StartADC()
{
    ADCON0bits.GO = 1;
}